import ast

from wtforms.fields import Field


def slider_widget(field, ul_class='', **kwargs):
    """widget for rendering a SliderField"""
    # THE BELOW IS JUST AN EXAMPLE, NOTHING TO DO WITH SLIDER
    kwargs.setdefault('type', 'checkbox')
    field_id = kwargs.pop('id', field.id)
    html = [u'<ul %s>' % html_params(id=field_id, class_=ul_class)]
    for value, label, checked in field.iter_choices():
        choice_id = u'%s-%s' % (field_id, value)
        options = dict(kwargs, name=field.name, value=value, id=choice_id)
        if checked:
            options['checked'] = 'checked'
        html.append(u'<li><input %s /> ' % html_options(**options))
        html.append(u'<label %s>%s</label></li>')
    html.append(u'</ul>')
    return u''.join(html)


class SliderField(Field):
    widget = slider_widget

    def __init__(self, id, min, max, value):
        self.id = id
        self.min = min
        self.max = max
        self.value = value

    def _value(self):
        return self.data

    def process_formdata(self, input):
        self.data = ast.literal_eval(input[0])



"""
Gleam: interactive visualizations in Python
"""

import os
import json
import urlparse
from collections import namedtuple

from flask import Flask, request, send_from_directory
from jinja2 import Environment, PackageLoader


GLEAM_STATIC_DIR = os.path.join(os.path.split(__file__)[0], "static")


class InputData(object):
    pass


class Page(object):
    @classmethod
    def add_flask(cls, app, path="/"):
        """Add this page to a Flask application at the given path"""

        env = Environment(loader=PackageLoader('gleam', 'templates'))

        panels_rendered = cls.input.render() + cls.output.render()

        # built-in static views
        @app.route('/gleam/<path:filename>')
        def custom_static(filename):
            return send_from_directory(GLEAM_STATIC_DIR, filename)

        # create main_view
        @app.route(path, methods=["GET"])
        def main_view():
            template = env.get_template('page.html')
            return template.render({'panels': panels_rendered})

        # create post view
        @app.route(path, methods=["POST"])
        def server_view():
            res = {}
            form_class = cls.input.form_class()
            form = form_class(request.form, csrf_enabled=False)
            if form.validate():
                # turn dictionary into an object
                d = InputData()
                d.__dict__.update(form.data)
                res = cls.output.refresh(d)
                return json.dumps({"changes": res})
            else:
                raise Exception("Form didn't validate")

    @classmethod
    def run(cls, debug=False, *args, **kwargs):
        """Create a Flask application with this single page"""
        app = Flask(cls.__name__)
        cls.add_flask(app)
        app.debug = debug
        app.run(*args, **kwargs)



from jinja2 import Environment, PackageLoader
import os
import hashlib
import random

import wtforms
from wtforms import Form
from wtforms.fields import Field
from wtforms.fields.core import UnboundField


class Panel(object):

    column_class = "col-md-4"

    def __init__(self):
        self.values = {}
        self.values["column_class"] = self.column_class

    def id(self):
        return self.name.replace(" ", "-").lower()

    def render(self):
        """Render the panel"""
        env = Environment(loader=PackageLoader('gleam', 'templates'))
        template = env.get_template(self.template_name)
        rendered = template.render(**self.values)
        return rendered


class InputPanel(Panel):
    """"An panel to display form inputs for the chart"""

    template_name = "input.html"

    def __init__(self):
        super(InputPanel, self).__init__()
        form_class = self.form_class()
        self.values["form"] = form_class(csrf_enabled=False)

    def form_class(self):
        """ Create an input form class with the fields in the panel class."""
        class InputForm(Form):
            extra = wtforms.fields.HiddenField()

        # get the fields
        for name, obj in self.__class__.__dict__.iteritems():
            if isinstance(obj, UnboundField):
                setattr(InputForm, name, obj)

        return InputForm


class TabPanel(Panel):

    template_name = "tabs.html"

    def __init__(self, tabs):
        self.tabs = tabs
        super(TabPanel, self).__init__()
        self.values["tabs"] = tabs

    def refresh(self, data):
        for tab in self.tabs:
            if tab.id() == data.extra:
                return tab.refresh(data)
        return {}


class PlotPanel(Panel):
    """A panel that contains a plot"""

    template_name = "plot.html"
    width = 700
    height = 500
    plotter = "ggplot"
    extension = "png"
    name = "plot"

    def __init__(self):
        super(PlotPanel, self).__init__()
        self.values["height"] = self.height
        self.values["width"] = self.width
        self.values["name"] = self.name
        self.plot_dir = os.path.join("static", "figures", self.name)
        if not os.path.exists(self.plot_dir):
            os.makedirs(self.plot_dir)

    def __call__(self, func):
        """Using as a decorator"""
        self.func = func
        self.args = inspect.getargspec(func).args
        return self

    def refresh(self, data):
        """Generate a new image, then tell the page to change the src"""
        h = hashlib.md5(str(data.__dict__)).hexdigest()
        print h

        outfile = os.path.join(self.plot_dir, h + "." + self.extension)

        if not os.path.exists(outfile):

            if self.plotter == "custom":
                kwargs["__outfile"] = outfile

            if self.plotter == "matplotlib":
                from matplotlib import pyplot as plt
                plt.clf()

            ret = self.plot(data)
            if ret is None:
                # don't change the plot at all
                return {self.name: {}}

            # after
            if self.plotter == "matplotlib":
                plt.savefig(outfile)
            elif self.plotter == "ggplot":
                from ggplot.utils import ggsave
                ggsave(outfile, ret)

        # turn into a URL, add a dummy param to avoid browser caching
        url = outfile.replace(os.path.sep, "/")
        url = url + "?dummy=" + str(random.random())

        return {self.name: {"src": url}}

    def __repr__(self):
        return self.render()



import os
from setuptools import setup

setup(name="gleam",
      author="David Robinson",
      author_email="dgrtwo@princeton.edu",
      description="Interactive visualization in Python",
      version="0.1.2",
      packages=["gleam"],
      package_dir={"gleam": "src/gleam"},
      package_data={"gleam": [os.path.join("templates", "*"),
                              os.path.join("static", "*", "*")]},
      install_requires=['Flask', 'Jinja2', 'WTForms']
      )
